/***************************************************************************

  joylinux.c

  Joystick devices component

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 1, or (at your option)
  any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************/

#include "config.h"

#ifdef OS_LINUX

#include <sys/types.h>
#include <sys/ioctl.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <fcntl.h>
#include <string.h>
#include <errno.h>

#include <linux/input.h>
#include <linux/joystick.h>

#include "joydev.h"

int joydev_init()
{
	return 1;
}

int joydev_opendevice(char *device, joyinfo *joystick)
{
	struct js_event ev;

	if ((joystick->fd = open (device, O_RDONLY)) < 0)
		return (-1);

	/* Some default values, joysticks got atleast 2 axes and 1 button */
	joystick->axises = 2;
	joystick->buttons = 1;
	snprintf(joystick->name, sizeof(joystick->name), "Unknown");
	snprintf(joystick->path, sizeof(joystick->path), device);

	ioctl(joystick->fd, JSIOCGAXES, joystick->axises);
	ioctl(joystick->fd, JSIOCGBUTTONS, joystick->buttons);
	ioctl(joystick->fd, JSIOCGNAME(MAX_NAME_LENGTH), joystick->name);

	fcntl(joystick->fd, F_SETFL, O_NONBLOCK);

	// flush all pending events
	while (read(joystick->fd, &ev, sizeof(struct js_event)) > 0) {}

	return (0);
}

void joydev_closedevice(joyinfo *joystick)
{
	if (!joystick)
		return;

	if (joystick->fd)
		close(joystick->fd);

	memset(joystick, 0, sizeof(joyinfo));
	return;
}

void joydev_getevents(joyinfo *joystick, joyevent *event)
{
	struct js_event ev;
	int type;

	if (!joystick || !event)
		return;

	if (read(joystick->fd, &ev, sizeof(struct js_event)) < 0)
	{
		if (errno == EAGAIN)
			goto no_event;
		else
		{
			event->type = ERROR;
			event->number = 0;
			event->value = errno;
			return;
		}
	}
	
	type = ev.type & ~JS_EVENT_INIT;

	if (type == JS_EVENT_BUTTON)
	{
		event->type = JOY_BUTTON;
		event->number = ev.number;
		event->value = ev.value;
		return;	
	}

	if (type == JS_EVENT_AXIS)
	{
		event->type = JOY_AXIS;
		event->number = ev.number;
		event->value = ev.value;
		return;	
	}

no_event:
	event->type = NO_EVENT;
}

#endif /* OS_LINUX */
